<?php

class Config
{
	var $conf;
	var $file;
	var $ttl;

	function Config( $file )
	{
		$this->conf = load_config( $file );
		$this->file = $file;

		// TTL(time to live) 1800 sec;
		$this->ttl  = 1800;
	}

	function &getInstance( $file = '' )
	{
		static $instance = NULL;

		if ( $instance === NULL ) {
			if ( $file && is_readable( $file ) && is_writable( $file ) ) {
				$instance = new Config( $file );

				// save config data at script end.
				register_shutdown_function( array( &$instance, 'save' ) );
			}
		}
		return $instance;
	}

	function createUniqID()
	{
		$curr = time();
		$ids  = $this->get( 'uniq_ids', array() );
		foreach ( $ids as $id => $time ) {
			if ( $time < $curr - $this->ttl ) {
				unset( $ids[$id] );
			}
			else {
				$uniq_id = $id;
			}
		}
		if ( count( $ids ) < 2 ) {
			if ( ! $ids || ( $curr - (int)( $this->ttl / 2 ) ) >= max( $ids ) ) {
				$uniq_id = md5( uniqid( $this->get( 'title', '' ) ) );
				$ids[$uniq_id] = time();
			}
		}
		$this->set( 'uniq_id',  $uniq_id );
		$this->set( 'uniq_ids', $ids );

		return $uniq_id;
	}

	function get( $key = NULL, $false = FALSE )
	{
		if ( $key === NULL ) {
			return $this->conf;
		}
		if ( isset( $this->conf[$key] ) ) {
			return $this->conf[$key];
		}
		return $false;
	}

	function getConfigFile()
	{
		return $this->file;
	}

	function isAuth()
	{
		$user = $this->get( 'user' );
		$pass = $this->get( 'pass' );
		if ( $user === FALSE && $pass === FALSE ) {
			return TRUE;
		}
		else {
			$_user = ifsetor( $_SERVER['PHP_AUTH_USER'], '' );
			$_pass = ifsetor( $_SERVER['PHP_AUTH_PW'],   '' );
			if ( $user === $_user && $pass === md5( $_pass ) ) {
				return TRUE;
			}
		}
		return FALSE;
	}

	function isCsrf()
	{
		$post =& Request::getInstance( '_POST', isset( $_POST ) ? $_POST : FALSE );
		if ( ! $post->get() ) {
			return FALSE;
		}
		$id  = $post->get( 'uniq_id', '' );
		$ids = $this->get( 'uniq_ids' );
		if ( isset( $ids[$id] ) && $ids[$id] > time() - $this->ttl ) {
			return FALSE;
		}
		return TRUE;
	}

	function save()
	{
		return save_config( $this->file, $this->conf );
	}

	function set( $key, $val )
	{
		$this->conf[$key] = $val;

		return TRUE;
	}

	function delete( $key )
	{
		if ( isset( $this->conf[$key] ) ) {
			unset( $this->conf[$key] );
		}
	}
}
